#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'html_tags/individual_tags/img.rb'
# =========================================================================== #
module HtmlTags

  require 'html_tags/opening_tag/opening_tag.rb'
  require 'html_tags/toplevel_methods/add.rb'
  require 'html_tags/toplevel_methods/newline_or_empty.rb'

  # ========================================================================= #
  # === HtmlTags.img
  #
  # The img() method creates an <img> tag.
  #
  # Documentation can be found at:
  #
  #   https://www.w3schools.com/tags/tag_img.asp
  #   https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img
  #
  # The "end result" will be a String such as this here:
  #
  #   <img src="foo.png" alt="Some image">
  #
  # Note that we make use of the block variant, by specifying a hash which
  # has a :src attribute.
  #
  # If the first argument to this method is a Hash then that Hash will
  # be checked for certain keys, such as :src or :css_class
  #
  # If the argument called the_id is a symbol such as 
  # :infer_the_id_from_the_filename then it will be evaluated in a
  # special manner.
  # ========================================================================= #
  def self.img(
      i         = 'foo.png',
      css_class = '',
      the_id    = '',
      css_style = ''
    )
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # === Handle Hashes given via block syntax next
      # ===================================================================== #
      if yielded.is_a? Hash
        if i.is_a? String
          i = { src: i } # Add this string as the :src key in that case.
          i.update(yielded)
        end
      end
    end
    if i.is_a? Hash
      # ===================================================================== #
      # === :css_style
      # ===================================================================== #
      if i.has_key? :css_style
        css_style = i.delete(:css_style)
      end
      # ===================================================================== #
      # === :css_class
      # ===================================================================== #
      if i.has_key? :css_class
        css_class = i.delete(:css_class)
      end
      # ===================================================================== #
      # === :id
      # ===================================================================== #
      if i.has_key? :id
        the_id = i.delete(:id)
      end
      # ===================================================================== #
      # === :src
      #
      # This entry point should come last, that is, after other checks were
      # already done. The reason for this is due to the deliberate
      # self-assignment being done here.
      # ===================================================================== #
      if i.has_key? :src
        i = i.delete(:src)
      end
    end
    case the_id # case tag
    # ======================================================================= #
    # === :drag_infer_the_id_from_the_filename
    # ======================================================================= #
    when :drag_infer_the_id_from_the_filename,
         :drag
      the_id = 'drag_'+File.basename(i).
               sub(/#{File.extname(i)}/,'')
    # ======================================================================= #
    # === :infer_the_id_from_the_filename
    # ======================================================================= #
    when :infer_the_id_from_the_filename
      the_id = File.basename(i).
               sub(/#{File.extname(i)}/,'')
    end
    return HtmlTags.add(
      opening_tag(__method__, css_class, the_id, css_style) {{ src: i }}+
      HtmlTags.newline_or_empty.to_s # i.to_s # Disabled adding the content. <img> tags do not have any content.
    )
  end

  # ========================================================================= #
  # === img
  # ========================================================================= #
  def img(
      i         = 'foo.png',
      css_class = '',
      the_id    = '',
      css_style = ''
    )
    HtmlTags.img(i, css_class, the_id, css_style)
  end

end

if __FILE__ == $PROGRAM_NAME
  # You could also include HtmlTags, then just use img() directly.
  # include HtmlTags
  puts HtmlTags.img(
    "studium/MASTER.png",
    css_style: 'margin-left:6px'
  )
end